import { Module, Provider } from '@nestjs/common';
import { CreateAuthenticationGrpcController } from './interface/grpc-controllers/create-authentication.grpc.controller';
import { CreateAuthenticationService } from './core/application/commands/create-authentication/create-authentication.service';
import { AuthenticationMapper } from './authentication.mapper';
import {
  AUTH_MESSAGE_PUBLISHER,
  AUTHENTICATION_CONFIGURATION_REPOSITORY,
  AUTHENTICATION_REPOSITORY,
  PASSWORD_VERIFIER,
  USERNAME_REPOSITORY,
} from './authentication.di-tokens';
import { AuthenticationRepository } from './infrastructure/authentication.repository';
import { PrismaService } from './infrastructure/prisma.service';
import { CqrsModule } from '@nestjs/cqrs';
import { DeleteAuthenticationGrpcController } from './interface/grpc-controllers/delete-authentication.grpc.controller';
import { DeleteAuthenticationService } from './core/application/commands/delete-authentication/delete-authentication.service';
import { MessageBrokerPublisher } from '@mobicoop/message-broker-module';
import { UsernameRepository } from './infrastructure/username.repository';
import { UsernameMapper } from './username.mapper';
import { AddUsernameGrpcController } from './interface/grpc-controllers/add-username.grpc.controller';
import { AddUsernameService } from './core/application/commands/add-username/add-username.service';
import { DeleteUsernameGrpcController } from './interface/grpc-controllers/delete-username.grpc.controller';
import { DeleteUsernameService } from './core/application/commands/delete-username/delete-username.service';
import { UpdateUsernameGrpcController } from './interface/grpc-controllers/update-username.grpc.controller';
import { UpdateUsernameService } from './core/application/commands/update-username/update-username.service';
import { UpdatePasswordGrpcController } from './interface/grpc-controllers/update-password.grpc.controller';
import { UpdatePasswordService } from './core/application/commands/update-password/update-password.service';
import { ValidateAuthenticationGrpcController } from './interface/grpc-controllers/validate-authentication.grpc.controller';
import { ValidateAuthenticationQueryHandler } from './core/application/queries/validate-authentication/validate-authentication.query-handler';
import { UserUpdatedMessageHandler } from './interface/message-handlers/user-updated.message-handler';
import { UserDeletedMessageHandler } from './interface/message-handlers/user-deleted.message-handler';
import { ConfigurationRepository } from '@mobicoop/configuration-module';
import { PasswordVerifier } from './infrastructure/password-verifier';

const grpcControllers = [
  CreateAuthenticationGrpcController,
  DeleteAuthenticationGrpcController,
  AddUsernameGrpcController,
  UpdateUsernameGrpcController,
  DeleteUsernameGrpcController,
  UpdatePasswordGrpcController,
  ValidateAuthenticationGrpcController,
];

const messageHandlers = [UserUpdatedMessageHandler, UserDeletedMessageHandler];

const commandHandlers: Provider[] = [
  CreateAuthenticationService,
  DeleteAuthenticationService,
  AddUsernameService,
  UpdateUsernameService,
  DeleteUsernameService,
  UpdatePasswordService,
];

const queryHandlers: Provider[] = [ValidateAuthenticationQueryHandler];

const mappers: Provider[] = [AuthenticationMapper, UsernameMapper];

const repositories: Provider[] = [
  {
    provide: AUTHENTICATION_REPOSITORY,
    useClass: AuthenticationRepository,
  },
  {
    provide: USERNAME_REPOSITORY,
    useClass: UsernameRepository,
  },
  {
    provide: AUTHENTICATION_CONFIGURATION_REPOSITORY,
    useClass: ConfigurationRepository,
  },
];

const messagePublishers: Provider[] = [
  {
    provide: AUTH_MESSAGE_PUBLISHER,
    useExisting: MessageBrokerPublisher,
  },
];

const orms: Provider[] = [
  PrismaService,
  {
    provide: PASSWORD_VERIFIER,
    useClass: PasswordVerifier,
  },
];

@Module({
  imports: [CqrsModule],
  controllers: [...grpcControllers],
  providers: [
    ...messageHandlers,
    ...commandHandlers,
    ...queryHandlers,
    ...mappers,
    ...repositories,
    ...messagePublishers,
    ...orms,
  ],
  exports: [
    PrismaService,
    AuthenticationMapper,
    UsernameMapper,
    AUTHENTICATION_REPOSITORY,
    USERNAME_REPOSITORY,
  ],
})
export class AuthenticationModule {}
