import { CommandHandler, ICommandHandler } from '@nestjs/cqrs';
import { Inject } from '@nestjs/common';
import {
  AggregateID,
  ConflictException,
  UniqueConstraintException,
} from '@mobicoop/ddd-library';
import {
  AUTHENTICATION_REPOSITORY,
  USERNAME_REPOSITORY,
} from '@modules/authentication/authentication.di-tokens';
import { AuthenticationRepositoryPort } from '../../ports/authentication.repository.port';
import { UsernameAlreadyExistsException } from '@modules/authentication/core/domain/authentication.errors';
import { AddUsernameCommand } from './add-username.command';
import { UsernameRepositoryPort } from '../../ports/username.repository.port';
import { UsernameEntity } from '@modules/authentication/core/domain/username.entity';

@CommandHandler(AddUsernameCommand)
export class AddUsernameService implements ICommandHandler {
  constructor(
    @Inject(AUTHENTICATION_REPOSITORY)
    private readonly authenticationRepository: AuthenticationRepositoryPort,
    @Inject(USERNAME_REPOSITORY)
    private readonly usernameRepository: UsernameRepositoryPort,
  ) {}

  async execute(command: AddUsernameCommand): Promise<AggregateID> {
    await this.authenticationRepository.findOneById(command.userId, {
      usernames: true,
    });
    try {
      const newUsername = await UsernameEntity.create({
        name: command.username.name,
        userId: command.userId,
        type: command.username.type,
      });
      await this.usernameRepository.insert(newUsername);
      return newUsername.id;
    } catch (error: any) {
      if (error instanceof ConflictException) {
        throw new UsernameAlreadyExistsException(error);
      }
      if (
        error instanceof UniqueConstraintException &&
        error.message.includes('username')
      ) {
        throw new UsernameAlreadyExistsException(error);
      }
      throw error;
    }
  }
}
