import { CommandHandler, ICommandHandler } from '@nestjs/cqrs';
import { Inject } from '@nestjs/common';
import {
  AggregateID,
  ConflictException,
  UniqueConstraintException,
} from '@mobicoop/ddd-library';
import { CreateAuthenticationCommand } from './create-authentication.command';
import {
  AUTHENTICATION_CONFIGURATION_REPOSITORY,
  AUTHENTICATION_REPOSITORY,
} from '@modules/authentication/authentication.di-tokens';
import { AuthenticationRepositoryPort } from '../../ports/authentication.repository.port';
import { AuthenticationEntity } from '@modules/authentication/core/domain/authentication.entity';
import {
  AuthenticationAlreadyExistsException,
  UsernameAlreadyExistsException,
} from '@modules/authentication/core/domain/authentication.errors';
import {
  Configurator,
  Domain,
  GetConfigurationRepositoryPort,
} from '@mobicoop/configuration-module';
import {
  AUTH_CONFIG_ENCRYPTION_ALGORITHM,
  AuthKeyTypes,
} from '@modules/authentication/authentication.constants';
import { EncryptionAlgorithm } from '@modules/authentication/core/domain/username.types';
import { PasswordEncrypter } from '@modules/authentication/infrastructure/password-encrypter';
import { PasswordEncrypterPort } from '../../ports/password-encrypter.port';

@CommandHandler(CreateAuthenticationCommand)
export class CreateAuthenticationService implements ICommandHandler {
  constructor(
    @Inject(AUTHENTICATION_REPOSITORY)
    private readonly authenticationRepository: AuthenticationRepositoryPort,
    @Inject(AUTHENTICATION_CONFIGURATION_REPOSITORY)
    private readonly configurationRepository: GetConfigurationRepositoryPort,
  ) {}

  async execute(command: CreateAuthenticationCommand): Promise<AggregateID> {
    const authConfigurator: Configurator =
      await this.configurationRepository.mget(Domain.AUTH, AuthKeyTypes);
    const passwordEncrypter: PasswordEncrypterPort = new PasswordEncrypter(
      authConfigurator.get<EncryptionAlgorithm>(
        AUTH_CONFIG_ENCRYPTION_ALGORITHM,
      ),
    );
    const authentication: AuthenticationEntity =
      await AuthenticationEntity.create({
        userId: command.userId,
        password: await passwordEncrypter.encrypt(command.password),
        usernames: command.usernames,
      });
    try {
      await this.authenticationRepository.insert(authentication);
      return authentication.getProps().userId;
    } catch (error: any) {
      if (error instanceof ConflictException) {
        throw new AuthenticationAlreadyExistsException(error);
      }
      if (
        error instanceof UniqueConstraintException &&
        error.message.includes('uuid')
      ) {
        throw new AuthenticationAlreadyExistsException(error);
      }
      if (
        error instanceof UniqueConstraintException &&
        error.message.includes('username')
      ) {
        throw new UsernameAlreadyExistsException(error);
      }
      throw error;
    }
  }
}
