import { PasswordVerifierPort } from '@modules/authentication/core/application/ports/password-verifier.port';
import { AuthenticationEntity } from '@modules/authentication/core/domain/authentication.entity';
import { CreateAuthenticationProps } from '@modules/authentication/core/domain/authentication.types';
import { AuthenticationDeletedDomainEvent } from '@modules/authentication/core/domain/events/authentication-deleted.domain-event';
import { PasswordUpdatedDomainEvent } from '@modules/authentication/core/domain/events/password-updated.domain-event';
import { Type } from '@modules/authentication/core/domain/username.types';

const createAuthenticationProps: CreateAuthenticationProps = {
  userId: '165192d4-398a-4469-a16b-98c02cc6f531',
  password: 'somePassword',
  usernames: [
    {
      type: Type.EMAIL,
      name: 'john.doe@email.com',
    },
  ],
};

const createAuthenticationPropsWith2Usernames: CreateAuthenticationProps = {
  userId: '165192d4-398a-4469-a16b-98c02cc6f531',
  password: 'somePassword',
  usernames: [
    {
      type: Type.EMAIL,
      name: 'john.doe@email.com',
    },
    {
      type: Type.PHONE,
      name: '+33611223344',
    },
  ],
};

const mockPasswordVerifier: PasswordVerifierPort = {
  verify: jest.fn().mockResolvedValueOnce(true).mockResolvedValueOnce(false),
};

describe('Authentication entity create', () => {
  it('should create a new authentication entity', async () => {
    const authenticationEntity: AuthenticationEntity =
      await AuthenticationEntity.create(createAuthenticationProps);
    expect(authenticationEntity.id).toBe(
      '165192d4-398a-4469-a16b-98c02cc6f531',
    );
    expect(authenticationEntity.domainEvents.length).toBe(1);
  });
  it('should create a new authentication entity with 2 usernames', async () => {
    const authenticationEntity: AuthenticationEntity =
      await AuthenticationEntity.create(
        createAuthenticationPropsWith2Usernames,
      );
    expect(authenticationEntity.id).toBe(
      '165192d4-398a-4469-a16b-98c02cc6f531',
    );
    expect(authenticationEntity.getProps().usernames.length).toBe(2);
    expect(authenticationEntity.domainEvents.length).toBe(1);
  });
});
describe('Authentication entity delete', () => {
  it('should delete an authentication entity', async () => {
    const authenticationEntity: AuthenticationEntity =
      await AuthenticationEntity.create(createAuthenticationProps);
    authenticationEntity.delete();
    expect(authenticationEntity.domainEvents.length).toBe(2);
    expect(authenticationEntity.domainEvents[1]).toBeInstanceOf(
      AuthenticationDeletedDomainEvent,
    );
  });
});
describe('Authentication password update', () => {
  it('should update the password of an authentication entity', async () => {
    const authenticationEntity: AuthenticationEntity =
      await AuthenticationEntity.create(createAuthenticationProps);
    const oldPassword: string = authenticationEntity.getProps().password;
    await authenticationEntity.updatePassword('@Br@ndN3wP@$$w0rd');
    expect(authenticationEntity.domainEvents.length).toBe(2);
    expect(authenticationEntity.domainEvents[1]).toBeInstanceOf(
      PasswordUpdatedDomainEvent,
    );
    expect(authenticationEntity.getProps().password).not.toBe(oldPassword);
  });
});
describe('Authentication password validation', () => {
  it('should validate a valid password', async () => {
    const authenticationEntity: AuthenticationEntity =
      await AuthenticationEntity.create(createAuthenticationProps);
    const result: boolean = await authenticationEntity.authenticate(
      'somePassword',
      mockPasswordVerifier,
    );
    expect(result).toBeTruthy();
  });
  it('should not validate an invalid password', async () => {
    const authenticationEntity: AuthenticationEntity =
      await AuthenticationEntity.create(createAuthenticationProps);
    const result: boolean = await authenticationEntity.authenticate(
      'someWrongPassword',
      mockPasswordVerifier,
    );
    expect(result).toBeFalsy();
  });
});
