import {
  Domain,
  Configurator,
  GetConfigurationRepositoryPort,
  KeyType,
} from '@mobicoop/configuration-module';
import {
  AggregateID,
  ConflictException,
  UniqueConstraintException,
} from '@mobicoop/ddd-library';
import { AUTH_CONFIG_ENCRYPTION_ALGORITHM } from '@modules/authentication/authentication.constants';
import {
  AUTHENTICATION_CONFIGURATION_REPOSITORY,
  AUTHENTICATION_REPOSITORY,
} from '@modules/authentication/authentication.di-tokens';
import { CreateAuthenticationCommand } from '@modules/authentication/core/application/commands/create-authentication/create-authentication.command';
import { CreateAuthenticationService } from '@modules/authentication/core/application/commands/create-authentication/create-authentication.service';
import { AuthenticationEntity } from '@modules/authentication/core/domain/authentication.entity';
import {
  AuthenticationAlreadyExistsException,
  UsernameAlreadyExistsException,
} from '@modules/authentication/core/domain/authentication.errors';
import { Type } from '@modules/authentication/core/domain/username.types';
import { CreateAuthenticationRequestDto } from '@modules/authentication/interface/grpc-controllers/dtos/create-authentication.request.dto';
import { Test, TestingModule } from '@nestjs/testing';

const createAuthenticationRequest: CreateAuthenticationRequestDto = {
  userId: '165192d4-398a-4469-a16b-98c02cc6f531',
  password: 'somePassword',
  usernames: [
    {
      name: 'john.doe@email.com',
      type: Type.EMAIL,
    },
  ],
};

const mockAuthenticationRepository = {
  insert: jest
    .fn()
    .mockImplementationOnce(() => ({}))
    .mockImplementationOnce(() => {
      throw new ConflictException('already exists');
    })
    .mockImplementationOnce(() => {
      throw new UniqueConstraintException('uuid');
    })
    .mockImplementationOnce(() => {
      throw new UniqueConstraintException('username');
    })
    .mockImplementationOnce(() => {
      throw new Error();
    }),
};

const mockConfigurationRepository: GetConfigurationRepositoryPort = {
  get: jest.fn(),
  mget: jest.fn().mockImplementation(
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    (domain: Domain, keyTypes: KeyType[]) => {
      switch (domain) {
        case Domain.AUTH:
          return new Configurator(Domain.AUTH, [
            {
              domain: Domain.AUTH,
              key: AUTH_CONFIG_ENCRYPTION_ALGORITHM,
              value: 'BCRYPT',
            },
          ]);
      }
    },
  ),
};

describe('Create Authentication Service', () => {
  let createAuthenticationService: CreateAuthenticationService;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        {
          provide: AUTHENTICATION_REPOSITORY,
          useValue: mockAuthenticationRepository,
        },
        {
          provide: AUTHENTICATION_CONFIGURATION_REPOSITORY,
          useValue: mockConfigurationRepository,
        },
        CreateAuthenticationService,
      ],
    }).compile();

    createAuthenticationService = module.get<CreateAuthenticationService>(
      CreateAuthenticationService,
    );
  });

  it('should be defined', () => {
    expect(createAuthenticationService).toBeDefined();
  });

  describe('execution', () => {
    const createAuthenticationCommand = new CreateAuthenticationCommand(
      createAuthenticationRequest,
    );
    AuthenticationEntity.create = jest.fn().mockReturnValue({
      id: '165192d4-398a-4469-a16b-98c02cc6f531',
      getProps: jest.fn().mockImplementation(() => ({
        userId: '165192d4-398a-4469-a16b-98c02cc6f531',
      })),
    });
    it('should create a new authentication', async () => {
      const result: AggregateID = await createAuthenticationService.execute(
        createAuthenticationCommand,
      );
      expect(result).toBe('165192d4-398a-4469-a16b-98c02cc6f531');
    });
    it('should throw a dedicated exception if Authentication already exists', async () => {
      await expect(
        createAuthenticationService.execute(createAuthenticationCommand),
      ).rejects.toBeInstanceOf(AuthenticationAlreadyExistsException);
    });
    it('should throw a dedicated exception if uuid already exists', async () => {
      await expect(
        createAuthenticationService.execute(createAuthenticationCommand),
      ).rejects.toBeInstanceOf(AuthenticationAlreadyExistsException);
    });
    it('should throw a dedicated exception if username already exists', async () => {
      await expect(
        createAuthenticationService.execute(createAuthenticationCommand),
      ).rejects.toBeInstanceOf(UsernameAlreadyExistsException);
    });
    it('should throw an error if something bad happens', async () => {
      await expect(
        createAuthenticationService.execute(createAuthenticationCommand),
      ).rejects.toBeInstanceOf(Error);
    });
  });
});
